<?php

namespace Drupal\contact_emails\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Connection;
use Drupal\contact_emails\ContactEmails;

/**
 * Class ContactEmailSettingsForm.
 *
 * @package Drupal\contact_emails\Form
 */
class ContactEmailSettingsForm extends FormBase {

  /**
   * Drupal\Core\Database\Connection definition.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Drupal\contact_emails\ContactEmails definition.
   *
   * @var \Drupal\contact_emails\ContactEmails;
   */
  protected $contactEmails;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $database, ContactEmails $contact_emails) {
    $this->database = $database;
    $this->contactEmails = $contact_emails;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('contact_emails.helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'contact_email_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $contact_form = '', $id = '') {

    if ($contact_form != 'personal') {

      $email = FALSE;
      if ($id) {
        $contact_emails = \Drupal::service('contact_emails.helper');
        $email = $contact_emails->getEmailById($id);
      }

      $form['id'] = [
        '#type' => 'hidden',
        '#value' => $id,
      ];
      $form['contact_form'] = [
        '#type' => 'hidden',
        '#value' => $contact_form,
      ];
      $form['subject'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Subject'),
        '#description' => $this->t('The subject of the email.'),
        '#maxlength' => 255,
        '#required' => TRUE,
        '#size' => 64,
        '#default_value' => ($email ? $email['subject'] : ''),
      ];

      $body = $this->getBody(($email['body'] ? $email['body'] : ''));
      $form['body'] = [
        '#type' => 'text_format',
        '#title' => $this->t('Body'),
        '#required' => TRUE,
        '#description' => $this->t('The body of the email. To add a variable, click within the above field and then click the "Browse available tokens" link below. You will find the fields of your forms within Contact Message.'),
        '#element_validate' => ['token_element_validate'],
        '#token_types' => ['contact_message'],
        '#default_value' => ($body ? $body['value'] : ''),
      ];
      if ($body['format']) {
        $form['body']['#format'] = $body['format'];
      }

      $form['token_help'] = array(
        '#theme' => 'token_tree_link',
        '#token_types' => ['contact_message'],
      );
      $form['recipient_is_submitter'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Send this email to the submitter of the form'),
        '#description' => $this->t('Check this box to send the email to the submitter of the form.'),
        '#default_value' => ($email ? $email['recipient_is_submitter'] : ''),
      ];
      $form['recipients'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Recipients'),
        '#description' => $this->t('Enter one or more recipients, separating multiple by commas.'),
        '#states' => [
          'visible' => [
            '#edit-recipient-is-submitter' => ['checked' => FALSE],
          ],
        ],
        '#default_value' => ($email ? $email['recipients'] : ''),
      ];
      $form['disabled'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Disable this email'),
        '#description' => $this->t('Check this box to disable sending of this email.'),
        '#default_value' => ($email ? $email['disabled'] : ''),
      ];

      $form['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Save email settings'),
      ];

    }
    else {

      $form['message'] = [
        '#plain_text' => $this->t('The Contact Emails module does not cover the personal contact form.'),
      ];
    }

    return $form;
  }

  /**
   * Get body potentially with format.
   */
  protected function getBody($body) {
    if ($body) {
      $body = unserialize($body);
      if (is_array($body)) {
        return $body;
      }
    }
    return [
      'value' => $body,
      'format' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    if (!$values['recipient_is_submitter'] && !$values['recipients']) {
      $form_state->setErrorByName('recipients', t('Please add at least one recipient.'));
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $values = $form_state->getValues();
    $fields = [
      'contact_form' => $values['contact_form'],
      'subject' => $values['subject'],
      'body' => serialize($values['body']),
      'recipients' => $values['recipients'],
      'recipient_is_submitter' => $values['recipient_is_submitter'],
      'disabled' => $values['disabled'],
    ];

    // Before we take any action, determine if we already have any
    // contact emails in place.
    $has_emails = $this->contactEmails->contactFormHasEmails($values['contact_form']);

    if ($values['id'] != 'new') {

      // Update existing.
      $this->database->update('contact_message_email_settings')
        ->fields($fields)
        ->condition('id', $values['id'])
        ->execute();
      drupal_set_message(t('The email has been successfully updated.'), 'status');
      $form_state->setRedirect('contact_emails.contact_emails_controller_list', [
        'contact_form' => $values['contact_form'],
      ]);

      // If this is the first email, warn that the Drupal Core contact email
      // has been disabled.
      if (!$has_emails) {
        drupal_set_message(t('The default contact email from the form settings has been disabled and your new email has replaced it.'), 'warning');
      }

    }
    else {

      // Create new.
      $values['id'] = $this->database->insert('contact_message_email_settings')
        ->fields($fields)
        ->execute();
      drupal_set_message(t('The email has been successfully created.'), 'status');
      $form_state->setRedirect('contact_emails.contact_emails_controller_list', [
        'contact_form' => $values['contact_form'],
      ]);

    }

    // Rebuild cache.
    $this->contactEmails->rebuildCache();
  }

}
